<?php
App::uses('EvInventoryAppModel', 'EvInventory.Model');
App::uses('InventoryActions', 'EvInventory.Lib');

/**
 * EvInventoryInventory Model
 *
 */
class Inventory extends EvInventoryAppModel {

	/**
	 * constructor - setup the dynamic relationships
	 *
	 */
	public function __construct($id = false, $table = null, $ds = null) {
		parent::__construct($id, $table, $ds);

		$relationships = Configure::read('EvInventory.InventoryBelongsTo');
		if (! empty($relationships)) {
			$belongsTo = array();
			foreach ($relationships as $key => $value) {
				$belongsTo[$key] = $value;
			}

			if (! empty($belongsTo)) {
				$this->bindModel(
					array(
						'belongsTo' => $belongsTo
					),
					false
				);
			}
		}
	}

	/**
	 * given the model name and the id get it's inventory
	 *
	 * @todo 	Refactor out at some point for better testable methods
	 * @param 	string 		$modelName 	The model name it belongs to
	 * @param 	int 		$modelId 	The models item id
	 * @return 	array|bool 	$inventory 	bool false on fail or inventory array
	 */
	public function getInventory($modelName, $modelId) {
		$inventoryContain = Configure::read('EvInventory.InventoryContains');

		if (empty($inventoryContain)) {
			$inventoryContain = array();

			$relationships = Configure::read('EvInventory.InventoryBelongsTo');

			foreach ($relationships as $key => $value) {
				if (is_array($value)) {
					$inventoryContain[] = $key;
				} else {
					$inventoryContain[] = $value;
				}
			}
		}

		$inventory = $this->find(
			'first',
			array(
				'conditions' => array(
					'Inventory.model' => $modelName,
					'Inventory.model_id' => $modelId
				),
				'contain' => $inventoryContain
			)
		);

		if (empty($inventory)) {
			return false;
		}

		$inventory['Inventory']['warning_action'] = json_decode($inventory['Inventory']['warning_action'], true);
		$inventory['Inventory']['oos_action'] = json_decode($inventory['Inventory']['oos_action'], true);

		return $inventory;
	}

	/**
	 * warning level reached, process the warning levl actions
	 *
	 * @param 	array 	$inventory 	The inventory row
	 * @return 	bool
	 */
	public function warningLevelReached($inventory) {
		if (! is_array($inventory['Inventory']['warning_action'])) {
			return true;
		}

		$ActionsLib = new InventoryActions($inventory, 'warning');

		foreach ($inventory['Inventory']['warning_action'] as $action) {
			if (method_exists($ActionsLib, $action)) {
				$ActionsLib->$action();
			}
		}

		return true;
	}

	/**
	 * out of stock, process the out of stock actions
	 *
	 * @param 	array 	$inventory 	The inventory row
	 * @return 	bool
	 */
	public function outOfStock($inventory) {
		if (! is_array($inventory['Inventory']['oos_action'])) {
			return true;
		}

		$ActionsLib = new InventoryActions($inventory, 'outofstock');

		foreach ($inventory['Inventory']['oos_action'] as $action) {
			if (method_exists($ActionsLib, $action)) {
				$ActionsLib->$action();
			}
		}

		return true;
	}
}
